@@ -18,6 +18,7 @@ const ExtensionSystem = imports.ui.extensionSystem;
 const ExtensionDownloader = imports.ui.extensionDownloader;
 const Keyboard = imports.ui.keyboard;
 const MessageTray = imports.ui.messageTray;
+const ModalDialog = imports.ui.modalDialog;
 const OsdWindow = imports.ui.osdWindow;
 const Overview = imports.ui.overview;
 const Panel = imports.ui.panel;
@@ -167,6 +168,16 @@ function _initializeUI() {
             overview.toggle();
     }));
 
+    global.display.connect('show-restart-message', function(display, message) {
+        showRestartMessage(message);
+        return true;
+    });
+
+    global.display.connect('restart', function() {
+        global.reexec_self();
+        return true;
+    });
+
     // Provide the bus object for gnome-session to
     // initiate logouts.
     EndSessionDialog.init();
@@ -233,8 +244,7 @@ function _loadDefaultStylesheet() {
  * Returns: A file path that contains the theme CSS,
  *          null if using the default
  */
-function getThemeStylesheet()
-{
+function getThemeStylesheet() {
     return _cssStylesheet;
 }
 
@@ -245,8 +255,7 @@ function getThemeStylesheet()
  *
  * Set the theme CSS file that the shell will load
  */
-function setThemeStylesheet(cssStylesheet)
-{
+function setThemeStylesheet(cssStylesheet) {
     _cssStylesheet = cssStylesheet;
 }
 
@@ -608,3 +617,28 @@ function queueDeferredWork(workId) {
         GLib.Source.set_name_by_id(_deferredTimeoutId, '[gnome-shell] _runAllDeferredWork');
     }
 }
+
+const RestartMessage = new Lang.Class({
+    Name: 'RestartMessage',
+    Extends: ModalDialog.ModalDialog,
+
+    _init : function(message) {
+        this.parent({ shellReactive: true,
+                      styleClass: 'restart-message',
+                      shouldFadeIn: false,
+                      destroyOnClose: true });
+
+        let label = new St.Label({ text: message });
+
+        this.contentLayout.add(label, { x_fill: false,
+                                        y_fill: false,
+                                        x_align: St.Align.MIDDLE,
+                                        y_align: St.Align.MIDDLE });
+        this.buttonLayout.hide();
+    }
+});
+
+function showRestartMessage(message) {
+    let restartMessage = new RestartMessage(message);
+    restartMessage.open();
+}
