@@ -1,9 +1,7 @@
 // -*- mode: js; js-indent-level: 4; indent-tabs-mode: nil -*-
 
 const Clutter = imports.gi.Clutter;
-const GLib = imports.gi.GLib;
 const Gtk = imports.gi.Gtk;
-const Gio = imports.gi.Gio;
 const Lang = imports.lang;
 const Shell = imports.gi.Shell;
 const Signals = imports.signals;
@@ -15,7 +13,6 @@ const GrabHelper = imports.ui.grabHelper;
 const Main = imports.ui.main;
 const Params = imports.misc.params;
 const Separator = imports.ui.separator;
-const Slider = imports.ui.slider;
 const Tweener = imports.ui.tweener;
 
 const Ornament = {
@@ -24,17 +21,6 @@ const Ornament = {
     CHECK: 2,
 };
 
-function _ensureStyle(actor) {
-    if (actor.get_children) {
-        let children = actor.get_children();
-        for (let i = 0; i < children.length; i++)
-            _ensureStyle(children[i]);
-    }
-
-    if (actor instanceof St.Widget)
-        actor.ensure_style();
-}
-
 function isPopupMenuItemVisible(child) {
     if (child._delegate instanceof PopupMenuSection)
         if (child._delegate.isEmpty())
@@ -106,6 +92,7 @@ const PopupBaseMenuItem = new Lang.Class({
 
         if (this._activatable) {
             this.actor.connect('button-release-event', Lang.bind(this, this._onButtonReleaseEvent));
+            this.actor.connect('touch-event', Lang.bind(this, this._onTouchEvent));
             this.actor.connect('key-press-event', Lang.bind(this, this._onKeyPressEvent));
         }
         if (params.reactive && params.hover)
@@ -132,6 +119,14 @@ const PopupBaseMenuItem = new Lang.Class({
         return Clutter.EVENT_STOP;
     },
 
+    _onTouchEvent: function (actor, event) {
+        if (event.type() == Clutter.EventType.TOUCH_END) {
+            this.activate(event);
+            return Clutter.EVENT_STOP;
+        }
+        return Clutter.EVENT_PROPAGATE;
+    },
+
     _onKeyPressEvent: function (actor, event) {
         let symbol = event.get_key_symbol();
 
