@@ -32,6 +32,7 @@ const SearchSystem = new Lang.Class({
         this._registerProvider(new AppDisplay.AppSearchProvider());
 
         this._searchSettings = new Gio.Settings({ schema_id: SEARCH_PROVIDERS_SCHEMA });
+        this._searchSettings.connect('changed::enabled', Lang.bind(this, this._reloadRemoteProviders));
         this._searchSettings.connect('changed::disabled', Lang.bind(this, this._reloadRemoteProviders));
         this._searchSettings.connect('changed::disable-external', Lang.bind(this, this._reloadRemoteProviders));
         this._searchSettings.connect('changed::sort-order', Lang.bind(this, this._reloadRemoteProviders));
@@ -242,8 +243,8 @@ const GridSearchResult = new Lang.Class({
             dragSource = icon.icon;
             content = { actor: actor, icon: icon };
         } else {
-            if (content._delegate && content._delegate.getDragActorSource)
-                dragSource = content._delegate.getDragActorSource();
+            if (content.getDragActorSource)
+                dragSource = content.getDragActorSource();
         }
 
         this.actor.set_child(content.actor);
@@ -266,7 +267,7 @@ const GridSearchResult = new Lang.Class({
 
         if (!dragSource)
             // not exactly right, but alignment problems are hard to notice
-            dragSource = content;
+            dragSource = content.actor;
         this._dragActorSource = dragSource;
     },
 
@@ -433,7 +434,7 @@ const ListSearchResults = new Lang.Class({
     },
 
     _setMoreIconVisible: function(visible) {
-        this.providerIcon.moreIcon.visible = true;
+        this.providerIcon.moreIcon.visible = visible;
     },
 
     _getMaxDisplayedResults: function() {
