@@ -283,11 +283,11 @@ const InputSourcePopup = new Lang.Class({
         this._select(this._selectedIndex);
     },
 
-    _keyPressHandler: function(keysym, backwards, action) {
+    _keyPressHandler: function(keysym, action) {
         if (action == this._action)
-            this._select(backwards ? this._previous() : this._next());
+            this._select(this._next());
         else if (action == this._actionBackward)
-            this._select(backwards ? this._next() : this._previous());
+            this._select(this._previous());
         else if (keysym == Clutter.Left)
             this._select(this._previous());
         else if (keysym == Clutter.Right)
@@ -366,13 +366,12 @@ const InputSourceIndicator = new Lang.Class({
         this._keybindingAction =
             Main.wm.addKeybinding('switch-input-source',
                                   new Gio.Settings({ schema_id: "org.gnome.desktop.wm.keybindings" }),
-                                  Meta.KeyBindingFlags.REVERSES,
+                                  Meta.KeyBindingFlags.NONE,
                                   Shell.KeyBindingMode.ALL,
                                   Lang.bind(this, this._switchInputSource));
         this._keybindingActionBackward =
             Main.wm.addKeybinding('switch-input-source-backward',
                                   new Gio.Settings({ schema_id: "org.gnome.desktop.wm.keybindings" }),
-                                  Meta.KeyBindingFlags.REVERSES |
                                   Meta.KeyBindingFlags.IS_REVERSED,
                                   Shell.KeyBindingMode.ALL,
                                   Lang.bind(this, this._switchInputSource));
@@ -473,8 +472,7 @@ const InputSourceIndicator = new Lang.Class({
 
         let popup = new InputSourcePopup(this._mruSources, this._keybindingAction, this._keybindingActionBackward);
         let modifiers = binding.get_modifiers();
-        let backwards = modifiers & Meta.VirtualModifier.SHIFT_MASK;
-        if (!popup.show(backwards, binding.get_name(), binding.get_mask()))
+        if (!popup.show(binding.is_reversed(), binding.get_name(), binding.get_mask()))
             popup.destroy();
     },
 
