@@ -19,6 +19,13 @@ const Tweener = imports.ui.tweener;
 const WindowMenu = imports.ui.windowMenu;
 
 const SHELL_KEYBINDINGS_SCHEMA = 'org.gnome.shell.keybindings';
+const MAXIMIZE_WINDOW_ANIMATION_TIME = 0.15;
+const UNMAXIMIZE_WINDOW_ANIMATION_TIME = 0.15;
+const MINIMIZE_WINDOW_ANIMATION_TIME = 0.2;
+const SHOW_WINDOW_ANIMATION_TIME = 0.15;
+const DIALOG_SHOW_WINDOW_ANIMATION_TIME = 0.1;
+const DESTROY_WINDOW_ANIMATION_TIME = 0.15;
+const DIALOG_DESTROY_WINDOW_ANIMATION_TIME = 0.1;
 const WINDOW_ANIMATION_TIME = 0.25;
 const DIM_BRIGHTNESS = -0.3;
 const DIM_TIME = 0.500;
@@ -874,12 +881,12 @@ const WindowManager = new Lang.Class({
         return !Main.overview.visible;
     },
 
-    _shouldAnimateActor: function(actor) {
+    _shouldAnimateActor: function(actor, types) {
         if (!this._shouldAnimate())
             return false;
-        let windowType = actor.meta_window.get_window_type();
-        return windowType == Meta.WindowType.NORMAL ||
-            windowType == Meta.WindowType.MODAL_DIALOG;
+
+        let type = actor.meta_window.get_window_type();
+        return types.indexOf(type) >= 0;
     },
 
     _removeEffect : function(list, actor) {
@@ -892,7 +899,10 @@ const WindowManager = new Lang.Class({
     },
 
     _minimizeWindow : function(shellwm, actor) {
-        if (!this._shouldAnimateActor(actor)) {
+        let types = [Meta.WindowType.NORMAL,
+                     Meta.WindowType.MODAL_DIALOG,
+                     Meta.WindowType.DIALOG];
+        if (!this._shouldAnimateActor(actor, types)) {
             shellwm.completed_minimize(actor);
             return;
         }
@@ -904,7 +914,7 @@ const WindowManager = new Lang.Class({
         if (actor.meta_window.is_monitor_sized()) {
             Tweener.addTween(actor,
                          { opacity: 0,
-                           time: WINDOW_ANIMATION_TIME,
+                           time: MINIMIZE_WINDOW_ANIMATION_TIME,
                            transition: 'easeOutQuad',
                            onComplete: this._minimizeWindowDone,
                            onCompleteScope: this,
@@ -936,8 +946,8 @@ const WindowManager = new Lang.Class({
                                scale_y: yScale,
                                x: xDest,
                                y: yDest,
-                               time: WINDOW_ANIMATION_TIME,
-                               transition: 'easeOutQuad',
+                               time: MINIMIZE_WINDOW_ANIMATION_TIME,
+                               transition: 'easeInExpo',
                                onComplete: this._minimizeWindowDone,
                                onCompleteScope: this,
                                onCompleteParams: [shellwm, actor],
@@ -953,7 +963,7 @@ const WindowManager = new Lang.Class({
             Tweener.removeTweens(actor);
             actor.set_scale(1.0, 1.0);
             actor.set_opacity(255);
-            actor.move_anchor_point_from_gravity(Clutter.Gravity.NORTH_WEST);
+            actor.set_pivot_point(0, 0);
 
             shellwm.completed_minimize(actor);
         }
@@ -1051,7 +1061,10 @@ const WindowManager = new Lang.Class({
             actor._windowType = type;
         }));
 
-        if (!this._shouldAnimateActor(actor)) {
+        let types = [Meta.WindowType.NORMAL,
+                     Meta.WindowType.DIALOG,
+                     Meta.WindowType.MODAL_DIALOG];
+        if (!this._shouldAnimateActor(actor, types)) {
             shellwm.completed_map(actor);
             return;
         }
@@ -1059,15 +1072,23 @@ const WindowManager = new Lang.Class({
         if (actor.meta_window.is_attached_dialog()) {
             /* Scale the window from the center of the parent */
             this._checkDimming(actor.get_meta_window().get_transient_for());
-            actor.set_scale(1.0, 0.0);
-            actor.set_pivot_point(0.5, 0.5);
+        }
+
+        switch (actor._windowType) {
+        case Meta.WindowType.NORMAL:
+            actor.set_pivot_point(0.5, 1.0);
+            actor.scale_x = 0.01;
+            actor.scale_y = 0.05;
+            actor.opacity = 0;
             actor.show();
             this._mapping.push(actor);
 
             Tweener.addTween(actor,
-                             { scale_y: 1,
-                               time: WINDOW_ANIMATION_TIME,
-                               transition: "easeOutQuad",
+                             { opacity: 255,
+                               scale_x: 1,
+                               scale_y: 1,
+                               time: SHOW_WINDOW_ANIMATION_TIME,
+                               transition: 'easeOutExpo',
                                onComplete: this._mapWindowDone,
                                onCompleteScope: this,
                                onCompleteParams: [shellwm, actor],
@@ -1075,15 +1096,20 @@ const WindowManager = new Lang.Class({
                                onOverwriteScope: this,
                                onOverwriteParams: [shellwm, actor]
                              });
-        } else {
-            /* Fade window in */
+            break;
+        case Meta.WindowType.MODAL_DIALOG:
+        case Meta.WindowType.DIALOG:
+            actor.set_pivot_point(0.5, 0.5);
+            actor.scale_y = 0;
             actor.opacity = 0;
             actor.show();
             this._mapping.push(actor);
 
             Tweener.addTween(actor,
                              { opacity: 255,
-                               time: WINDOW_ANIMATION_TIME,
+                               scale_x: 1,
+                               scale_y: 1,
+                               time: DIALOG_SHOW_WINDOW_ANIMATION_TIME,
                                transition: 'easeOutQuad',
                                onComplete: this._mapWindowDone,
                                onCompleteScope: this,
@@ -1092,6 +1118,10 @@ const WindowManager = new Lang.Class({
                                onOverwriteScope: this,
                                onOverwriteParams: [shellwm, actor]
                              });
+            break;
+        default:
+            shellwm.completed_map(actor);
+            return;
         }
     },
 
@@ -1099,7 +1129,11 @@ const WindowManager = new Lang.Class({
         if (this._removeEffect(this._mapping, actor)) {
             Tweener.removeTweens(actor);
             actor.opacity = 255;
+            actor.set_pivot_point(0, 0);
             actor.scale_y = 1;
+            actor.scale_x = 1;
+            actor.translation_y = 0;
+            actor.translation_x = 0;
             shellwm.completed_map(actor);
         }
     },
@@ -1122,30 +1156,25 @@ const WindowManager = new Lang.Class({
                                                              });
         }
 
-        if (!this._shouldAnimateActor(actor)) {
+        let types = [Meta.WindowType.NORMAL,
+                     Meta.WindowType.DIALOG,
+                     Meta.WindowType.MODAL_DIALOG];
+        if (!this._shouldAnimateActor(actor, types)) {
             shellwm.completed_destroy(actor);
             return;
         }
 
-        this._destroying.push(actor);
-
-        if (window.is_attached_dialog()) {
-            let parent = window.get_transient_for();
-            this._checkDimming(parent, window);
-
-            actor.set_scale(1.0, 1.0);
+        switch (actor._windowType) {
+        case Meta.WindowType.NORMAL:
             actor.set_pivot_point(0.5, 0.5);
-            actor.show();
-
-            actor._parentDestroyId = parent.connect('unmanaged', Lang.bind(this, function () {
-                Tweener.removeTweens(actor);
-                this._destroyWindowDone(shellwm, actor);
-            }));
+            this._destroying.push(actor);
 
             Tweener.addTween(actor,
-                             { scale_y: 0,
-                               time: WINDOW_ANIMATION_TIME,
-                               transition: "easeOutQuad",
+                             { opacity: 0,
+                               scale_x: 0.8,
+                               scale_y: 0.8,
+                               time: DESTROY_WINDOW_ANIMATION_TIME,
+                               transition: 'easeOutQuad',
                                onComplete: this._destroyWindowDone,
                                onCompleteScope: this,
                                onCompleteParams: [shellwm, actor],
@@ -1153,9 +1182,37 @@ const WindowManager = new Lang.Class({
                                onOverwriteScope: this,
                                onOverwriteParams: [shellwm, actor]
                              });
+            break;
+        case Meta.WindowType.MODAL_DIALOG:
+        case Meta.WindowType.DIALOG:
+            actor.set_pivot_point(0.5, 0.5);
+            this._destroying.push(actor);
+
+            if (window.is_attached_dialog()) {
+                let parent = window.get_transient_for();
+                this._checkDimming(parent, window);
+                actor._parentDestroyId = parent.connect('unmanaged', Lang.bind(this, function () {
+                    Tweener.removeTweens(actor);
+                    this._destroyWindowDone(shellwm, actor);
+                }));
+            }
+
+            Tweener.addTween(actor,
+                             { scale_y: 0,
+                               time: DIALOG_DESTROY_WINDOW_ANIMATION_TIME,
+                               transition: 'easeOutQuad',
+                               onComplete: this._destroyWindowDone,
+                               onCompleteScope: this,
+                               onCompleteParams: [shellwm, actor],
+                               onOverwrite: this._destroyWindowDone,
+                               onOverwriteScope: this,
+                               onOverwriteParams: [shellwm, actor]
+                             });
+            break;
+        default:
+            shellwm.completed_destroy(actor);
             return;
         }
-        shellwm.completed_destroy(actor);
     },
 
     _destroyWindowDone : function(shellwm, actor) {
@@ -1328,9 +1385,7 @@ const WindowManager = new Lang.Class({
 
         let tabPopup = new AltTab.AppSwitcherPopup();
 
-        let modifiers = binding.get_modifiers();
-        let backwards = modifiers & Meta.VirtualModifier.SHIFT_MASK;
-        if (!tabPopup.show(backwards, binding.get_name(), binding.get_mask()))
+        if (!tabPopup.show(binding.is_reversed(), binding.get_name(), binding.get_mask()))
             tabPopup.destroy();
     },
 
@@ -1341,16 +1396,12 @@ const WindowManager = new Lang.Class({
 
         let tabPopup = new AltTab.WindowSwitcherPopup();
 
-        let modifiers = binding.get_modifiers();
-        let backwards = modifiers & Meta.VirtualModifier.SHIFT_MASK;
-        if (!tabPopup.show(backwards, binding.get_name(), binding.get_mask()))
+        if (!tabPopup.show(binding.is_reversed(), binding.get_name(), binding.get_mask()))
             tabPopup.destroy();
     },
 
     _startA11ySwitcher : function(display, screen, window, binding) {
-        let modifiers = binding.get_modifiers();
-        let backwards = modifiers & Meta.VirtualModifier.SHIFT_MASK;
-        Main.ctrlAltTabManager.popup(backwards, binding.get_name(), binding.get_mask());
+        Main.ctrlAltTabManager.popup(binding.is_reversed(), binding.get_name(), binding.get_mask());
     },
 
     _toggleAppMenu : function(display, screen, window, event, binding) {
