@@ -34,6 +34,8 @@ extern GType gnome_shell_plugin_get_type (void);
 #define SHELL_DBUS_SERVICE "org.gnome.Shell"
 #define MAGNIFIER_DBUS_SERVICE "org.gnome.Magnifier"
 
+#define OVERRIDES_SCHEMA "org.gnome.shell.overrides"
+
 #define WM_NAME "GNOME Shell"
 #define GNOME_WM_KEYBINDINGS "Mutter,GNOME Shell"
 
@@ -167,6 +169,26 @@ shell_dbus_init (gboolean replace)
   g_object_unref (session);
 }
 
+static void
+shell_prefs_init (void)
+{
+  ShellGlobal *global = shell_global_get ();
+  GSettings *settings = shell_global_get_overrides_settings (global);
+  char **keys, **k, *schema_id;
+
+  if (!settings)
+    return;
+
+  g_object_get (G_OBJECT (settings), "schema-id", &schema_id, NULL);
+
+  keys = g_settings_list_keys (settings);
+  for (keys = k = g_settings_list_keys (settings); *k; k++)
+    meta_prefs_override_preference_schema (*k, schema_id);
+
+  g_strfreev (keys);
+  g_free (schema_id);
+}
+
 static void
 shell_introspection_init (void)
 {
@@ -434,6 +456,8 @@ main (int argc, char **argv)
 
   _shell_global_init ("session-mode", session_mode, NULL);
 
+  shell_prefs_init ();
+
   ecode = meta_run ();
 
   if (g_getenv ("GNOME_SHELL_ENABLE_CLEANUP"))
